# 后端日志规范

## 版本修订

1. **2015-10-14** 创建
2. **2015-10-20** SYS中添加错误号

## 适用范围

所有运行着的线上django服务的业务日志，包括但不限于`gaia`, `backend`, `ship`, `flag-ship`, `ascle`。并希望之后所有的服务的业务日志统一遵守这一规范。

## 日志路径和文件名

原始日志应存放于`/data/log/${SRV_NAME}/`文件夹下，其中`${SRV_NAME}`为服务名。可以根据需求拆分子文件夹。

通常情况下，日志文件名应该以模块名为前缀。对于不同日志等级的文件名后缀如下：
*   `DEBUG`: 以`.log.debug`为后缀，如`gaia.log.debug`。
*   `INFO`: 以`.log`为后缀，如`gaia.log`。
*   `WARNING`: 以`.log.warn`为后缀，如`gaia.log.warn`。
*   `ERROR`: 以`.log.err`为后缀，如`gaia.log.err`。
对于非当天的日志(logrotate的日志)，需要加上日期后缀`.YYYY-MM-DD`，如`gaia.log.warn.2015-10-14`为2015-10-14的本机gaia WARNING日志。

## 日志编码

业务日志以`JSON`作为编码。一行一个`JSON object`，以换行符`\n`分隔开。

## 日志示例

    {"SYS": {"http_verb":"POST", "action": "/api/topic/create", "log_host": "10.127.158.29", "cl_id": "861342028123987123", "session_id": "cx123nfglxkdzlfjadwrsdnvsd34z", "cl_type": "ios", "cl_ver": "5.2.1", "channel": "mingxingyichu", "user_ip":"106.187.42.233"}, "LEVEL": "INFO", "APP": {"minami": "kotori", "hello": "world"}, "TIMER": {"total": 123.456, "get_user_from_context": 78.9}, "LOGID": "", "TIME": "2015-10-14T17:13:20+08:00", "MODULE":"gaia"}

## 日志字段

所有日志字段的key要求只能使用以下字符`0-9`, `a-z`, `A-Z`, `-`和`_`。

1.  必打字段

    所有日志必须包含以下字段
    *   `LEVEL`: 日志等级。`DEBUG`, `INFO`, `WARNING`, `ERROR`等。
    
        -   `DEBUG`
            记录调试信息，用于线上排查故障。
            
        -   `INFO`
            记录正常的业务信息。`INFO`日志对于一个请求打印且只打印一条日志，包含下述的`SYS`, `APP`, `TIMER`字段。
            
        -   `WARNING`
            系统存在问题但不影响系统正常使用, 可关注此问题并在下一个release里修复。
            
        -   `ERROR`
            系统存在影响用户正常使用的严重问题，需要立即上线修复。
        
    *   `LOGID`: trace用，暂时空缺，填空字符串。
    *   `TIME`: 日志记录时间。使用[ISO8601标准](https://en.wikipedia.org/wiki/ISO_8601)，要求包含时区`+08:00`。根据需要决定是否精确到毫秒。完整例子如`2015-10-14T17:13:00+08:00`。
    *   `MODULE`: 打印日志的模块。

2.  `SYS`字段

    `SYS`字段包含一些全局性的或系统级别的信息，只要有就必须打。包含以下子字段
    *   `http_verb`: 请求的HTTP VERB(`GET`/`POST`/`DELETE`)
    *   `action`: 请求的URL(去掉GET参数)。
    *   `cl_type`: 请求来源端类型，`ios`/`android`/`pc`/`m`。
    *   `cl_ver`: 客户端版本号，`pc`不需要填写，m站除了native内嵌h5外不需要填写。
    *   `cl_id`: 设备标识，android取`device_id`，ios取`idfa`。
    *   `channel`: 请求渠道，客户端取包名即`channel`字段。pc和m站需要具体分析，暂不填写。
    *   `session_id`: 用户登陆后必须填写。否则不填写。
    *   `log_host`: 打印该日志所在机器的内网IP。
    *   `user_ip`: 用户的外网IP。
    *   `errno`: 错误号。无错误填0。出错或出现未知异常时要求填写非0值。
    

3.  `APP`字段

    `APP`字段包含一些业务性的信息，通常用于统计使用。例如下订单操作的订单金额，发私信的收信用户id等。部分字段经讨论若存在普适性可能考虑迁移至`SYS`部分。

4.  `TIMER`字段

    `TIMER`字段包括所有的请求耗时信息，值以毫秒为单位。必须包括`total`子字段：
    *   `total`: 该请求总耗时。

